#!/bin/sh

log="/var/log/senseshield_install.log"
install_path=/opt/senseshield
SENSEPATH=$install_path/senseshield

export PATH="$PATH:/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin"
 
write_log()
{	
    echo "[$0:"`date "+%Y-%m-%d %X %s"`"]" $1
    # echo "[$0:"`date "+%Y-%m-%d %X %s"`"]" $1  >> $log
}

getPathWithProcess()
{
    # 1. 检查进程是否存在
    ps -fe | grep $1 | grep -v "grep" >/dev/null 2>&1
    
    if [ $? -ne 0 ]
    then
        # p进程不存在
        return 1
    fi
    
    # 2. 获取程序路径
    pid=`ps axu | grep "senseshield" | grep -v "grep" | awk '{print $2}'`
    exe_path=`ls -l /proc/${pid} | grep "exe ->" | grep -v "grep" | awk '{print $NF}'`
    echo $(dirname $exe_path)
}

getPathWithCommand()
{
    # 1. 判断命令是否存在
    command -v $1 >/dev/null 2>&1
    
    if [ $? -ne 0 ]
    then
        # 命令不存在
        return 1
    fi

    # 2. 根据命令获取路径
    command_path=`whereis $1 | awk '{print $NF}'`
    exe_path=`ll $command_path | awk '{print $NF}'`
    echo $(dirname $exe_path)
}

preInstall()
{
    # 1. 尝试获取旧程序路径 
    # 优先级 ： 根据进程获取 -> 根据命令获取 -> /usr/lib/senseshield
    program_path=`getPathWithProcess "senseshield"`
    if [ $? -ne 0 ]
    then
        program_path=`getPathWithCommand "ssclt"`
    fi
    if [ $? -ne 0 ]
    then
        if [ -e "/usr/lib/senseshield/senseshield" ]
        then
            program_path="/usr/lib/senseshield"
        else
            program_path=""
        fi
    fi

    # 2. 判断程序是否已经安装 如果存在进行卸载
    # 1.> 判断是否存在dpkg命令
    command -v dpkg >/dev/null 2>&1
    if [ $? -eq 0 ]
    then
        dpkg -s senseshield >/dev/null 2>&1
        if [ $? -eq 0 ]
        then
            dpkg -P senseshield
        fi
    fi

    # 2.> 判断是否存在rpm命令
    command -v rpm >/dev/null 2>&1
    if [ $? -eq 0 ]
    then
        rpm -qi senseshield >/dev/null 2>&1
        if [ $? -eq 0 ]
        then
            rpm -e senseshield
        fi
    fi
    
    # 3.> 判断extra.sh文件是否存在 （压缩包安装）
    if [ -e /usr/lib/senseshield/extra.sh ]
    then
        /usr/lib/senseshield/extra.sh uninstall
    elif [ -e /opt/senseshield/extra.sh ]
    then
        /opt/senseshield/extra.sh uninstall
    fi


    # 3. 拷贝配置
    if [ -n "$program_path" ] && [ "$program_path" != "$install_path" ]
    then
        mv $program_path /opt
    fi

}

install()
{
    # 1. 检查安装包是否与安装环境匹配
    if [ $(getconf WORD_BIT) = '32' ] && [ $(getconf LONG_BIT) = '64' ] ; then
        install_file_path=x64
    else
        install_file_path=x86
    fi

    # 2. 如果存在旧版 则先进行卸载操作 并拷贝配置文件
    preInstall
    
	# 3. 创建文件夹
    echo "[info] mkdir"
	mkdir -p $install_path >> $log
	mkdir -p /lib/systemd/system  >> $log
	
    echo "[info] copy files"
    version=`cat $install_file_path/version`
	write_log "Version:"$version
    
    cp $install_file_path/* $install_path -r 
	cp $install_path/80-elite5.rules /etc/udev/rules.d/80-elite5.rules
    
    # 检查当前安装包是否打包 WebServer 组件
    pack_web_server=0
    if [ -f "$install_path/rootCA.cer" ] ; then
        pack_web_server=1
    fi
    
    if [ ! -f "/bin/systemctl" ] ; then
        if [ ! -f "/usr/sbin/iptables" ] ; then
            # 不支持防火墙（iptables），故在安装时删除服务配置中的防火墙相关命令，避免安装时提示错误信息。
            echo "[info] install delete file($install_path/config/senseshieldd) iptables cmd."
            sed -i '/addport/d' $install_path/config/senseshieldd
            sed -i '/delport/d' $install_path/config/senseshieldd
            
            echo "[info] install delete file($install_path/config/senseshield.service) iptables cmd."
            sed -i '/addport/d' $install_path/config/senseshield.service
            sed -i '/delport/d' $install_path/config/senseshield.service
        fi
    fi
    
    if [ -f "/bin/systemctl" ] ; then
        cp $install_path/config/senseshield.service  /lib/systemd/system/senseshield.service
    else
        cp $install_path/config/senseshieldd  /etc/init.d/senseshieldd
    fi
 
    # 创建软连接
    ln -s $install_path/ssclt /usr/bin/ssclt
    ln -s $install_path/sscfg /usr/bin/sscfg

	# 4. 添加权限
	write_log "add permission "
	chmod 755 $install_path/version
	chmod 755 $install_path/ss_config.xml
	chmod 755 $install_path/etc/ss/authconfig.xml
    chmod 755 $install_path/etc/common/ss_cloud.xml
	chmod 755 $install_path/extra.sh
    
    if [ $pack_web_server = 1 ] ; then
        chmod 755 $install_path/rootCA.cer
        chmod 755 $install_path/server.key
        chmod 755 $install_path/server.pem
        chmod 755 $install_path/virbox_web_server_cert_mgr.sh
    fi
	
	chmod 744 $install_path/senseshield
	chmod 755 $install_path/ssclt
	chmod 755 $install_path/sscfg
    
    chmod 755 /usr/bin/ssclt
	chmod 755 /usr/bin/sscfg
    
    if [ -e $install_path/lib ] ; then
        chmod -R 755 $install_path/lib
    fi
	
	# 5. 添加服务
	write_log "add service"
	#deb安装包会将脚本所在的目录文件全部安装包到系统里 我们的服务管理根据高低版本支持会有两套解决方法
	#所以需要互相删除不需要的配置文件，不删除则会产生服务命令不起作用的问题
    if [ -f "/bin/systemctl" ] ; then
        echo "daemon-reload " >> $log
        rm -f /etc/init.d/senseshieldd
        chmod 644 /lib/systemd/system/senseshield.service
        cd /lib/systemd/system
        systemctl daemon-reload
        echo `systemctl enable senseshield.service 2>&1` >> $log
        systemctl start senseshield.service
    else
        rm -f /lib/systemd/system/senseshield.service
        chmod 755 /etc/init.d/senseshieldd
        cd /etc/rc2.d;  ln -sf ../init.d/senseshieldd ./S99senseshieldd;
        cd /etc/rc3.d;  ln -sf ../init.d/senseshieldd ./S99senseshieldd;
        cd /etc/rc4.d;  ln -sf ../init.d/senseshieldd ./S99senseshieldd;
        cd /etc/rc5.d;  ln -sf ../init.d/senseshieldd ./S99senseshieldd;
        /etc/init.d/senseshieldd start
    fi
    
    # 5.1 导入 Web Server 证书
    if [ $pack_web_server = 1 ] ; then
        $install_path/virbox_web_server_cert_mgr.sh install $install_path
    fi

	#6. 启动同步服务（SSSync）
    # if [ "$2" = "linux" ] ; then
        # /usr/lib/senseshield/sssync
        # if [ -f "/etc/rc.d/rc.local" ] ; then
            # sed -i '/sssync/d' /etc/rc.d/rc.local
            # line_number=`grep -nv "#" /etc/rc.d/rc.local | head -1 | awk -F ":" '{print $1}'`
            # if [ x$line_number = x ] ; then
                # line_number="1"
                # echo "" > /etc/rc.d/rc.local
            # fi
            # sed -i "${line_number}i\/usr/lib/senseshield/sssync" /etc/rc.d/rc.local
        # fi
        # if [ ! -f "/etc/rc.local" ] ; then
            # touch /etc/rc.local
            # echo "#!/bin/sh" >> /etc/rc.local
            # echo "/usr/lib/senseshield/sssync" >> /etc/rc.local
            # echo "exit 0" >> /etc/rc.local
            # chmod 755 /etc/rc.local
            # if [ -f "/etc/rc.d/rc.local" ] ; then
                # chmod 755 /etc/rc.d/rc.local
            # fi
        # else
            # sed -i '/sssync/d' /etc/rc.local
            # line_number=`grep -nv "#" /etc/rc.local | head -1 | awk -F ":" '{print $1}'`
            # if [ x$line_number = x ] ; then
            # line_number="1"
            # echo "" > /etc/rc.local
            # fi
            # sed -i "${line_number}i\/usr/lib/senseshield/sssync" /etc/rc.local
            # chmod +x /etc/rc.local
            # if [ -f "/etc/rc.d/rc.local" ] ; then
                # chmod +x /etc/rc.d/rc.local
            # fi
        # fi
    # fi
	ret=`ps -ef|grep "$SENSEPATH" |grep -v "grep"`
    ret2=`ps|grep "$SENSEPATH" |grep -v "grep"`     # 部分ARM系统 ps 参数不支持参数
	if [ x"$ret" != x -o x"$ret2" != x ] ; then
		echo "install success."
	else 
		echo "install failed. senseshield service is not running."
	fi
    if [ "$1" = "RPM" ] ; then
        rm -rf x86/ x64/ config/
    fi
	write_log "install end"

}

uninstall()
{
	write_log "uninstall"
	version=`cat $install_path/version`
	write_log "Version:"$version

	# write_log "kill sssync"
	# ret=`ps -ef|grep sssync |grep -v "grep"`
    # ret2=`ps|grep sssync |grep -v "grep"`
	# if [ x"$ret" != x -o x"$ret2" != x ]    # -o(or)
	# then
		# kill -9 $(pidof sssync)
	# fi

	# sed -i '/sssync/d' /etc/rc.local
	# if [ -f "/etc/rc.d/rc.local" ] ; then
		# sed -i '/sssync/d' /etc/rc.d/rc.local
	# fi
	write_log "remove service"
    
    # 检查当前安装包是否打包 WebServer 组件
    if [ -f "$install_path/rootCA.cer" ] ; then
        # 卸载 Web Server 证书
        $install_path/virbox_web_server_cert_mgr.sh uninstall
    fi
    
    if [ -f "/bin/systemctl" ] ; then
        cd /lib/systemd/system
        systemctl stop senseshield.service
        echo  `systemctl disable senseshield.service 2>&1`  >> $log
        rm -f /lib/systemd/system/senseshield.service >> $log
    else
        /etc/init.d/senseshieldd stop
        rm -f /etc/init.d/senseshieldd
        rm -f /etc/rc2.d/S99senseshieldd
        rm -f /etc/rc3.d/S99senseshieldd
        rm -f /etc/rc4.d/S99senseshieldd
        rm -f /etc/rc5.d/S99senseshieldd
    fi
    
	write_log "delete files"

    # rules
	rm -f /etc/udev/rules.d/80-elite5.rules
    # ln
    rm -f /usr/bin/ssclt
	rm -f /usr/bin/sscfg
    
    # if [ "$2" = "linux" ] ; then
        # rm -f /usr/lib/senseshield/sssync
        # rm -f /usr/lib/senseshield/sssynccfg.ini
    # fi
    
    rm -rf $install_path/etc/common
    rm -f $install_path/etc/ss/authconfig.xml
    rm -rf $install_path/etc/ss_service

    rm -rf $install_path/config
    rm -rf $install_path/lib

	rm -f $install_path/senseshield
    rm -f $install_path/ssclt
	rm -f $install_path/sscfg
    rm -f $install_path/80-elite5.rules
    rm -f $install_path/authconfig.xml
    rm -f $install_path/sssynccfg.ini
    # config
	rm -f $install_path/ss_config.xml
    rm -f $install_path/extra.sh	
	rm -f $install_path/version
    # web server
    if [ -e $install_path/rootCA.cer ] ; then
        rm -f $install_path/rootCA.cer
        rm -f $install_path/server.key
        rm -f $install_path/server.pem
        rm -f $install_path/virbox_web_server_cert_mgr.sh
    fi
    # rules
	rm -f /etc/udev/rules.d/80-elite5.rules

	echo "uninstall success"
}

if grep -Eqi "CentOS" /etc/issue || grep -Eq "CentOS" /etc/*-release; then
    LINUXNAME='CentOS'
elif grep -Eqi "Ubuntu" /etc/issue || grep -Eq "Ubuntu" /etc/*-release; then
    LINUXNAME='Ubuntu'
elif grep -Eqi "Debian" /etc/issue || grep -Eq "Debian" /etc/*-release; then
    LINUXNAME='Debian'
elif grep -Eqi "Fedora" /etc/issue || grep -Eq "Fedora" /etc/*-release; then
    LINUXNAME='Fedora'
else
    LINUXNAME='UnKown'
fi

iptables_add_port()
{
    if [ -f "/usr/bin/firewall-cmd" -a $LINUXNAME != "Fedora" ];then
        firewall-cmd --zone=public --add-port=10334/tcp  >> $log
        firewall-cmd --zone=public --add-port=10335/udp  >> $log
        firewall-cmd --zone=public --add-port=10336/udp  >> $log
    elif [ -f "/usr/sbin/ufw" ];then
        ufw allow 10334/tcp >> $log
        ufw allow 10335/udp >> $log
        ufw allow 10336/udp >> $log	
    else
        /sbin/iptables -I INPUT -p tcp --dport 10334  -j ACCEPT >> $log
        /sbin/iptables -I INPUT -p udp --dport 10335  -j ACCEPT >> $log
        /sbin/iptables -I INPUT -p udp --dport 10336  -j ACCEPT >> $log
    fi
}

iptables_del_port()
{
    if [ -f "/usr/bin/firewall-cmd"  -a $LINUXNAME != "Fedora" ];then
        firewall-cmd --zone=public --remove-port=10334/tcp  >> $log
        firewall-cmd --zone=public --remove-port=10335/udp  >> $log
        firewall-cmd --zone=public --remove-port=10336/udp  >> $log
    elif [ -f "/usr/sbin/ufw" ];then
        ufw delete allow 10334/tcp >> $log
        ufw delete allow 10335/udp >> $log
        ufw delete allow 10336/udp >> $log	
    else
        /sbin/iptables -D INPUT -p tcp --dport 10334  -j ACCEPT >> $log
        /sbin/iptables -D INPUT -p udp --dport 10335  -j ACCEPT >> $log
        /sbin/iptables -D INPUT -p udp --dport 10336  -j ACCEPT >> $log
    fi
}

pkg_type=linux    # 支持类型：linux, arm，默认为 linux，SS打包时由 linuxss.sh 修改此设置。
#echo '[debug] extra.sh: pkg_type='$pkg_type

if [ "$1" = "install" ] ; then
    install $2 $pkg_type
elif [ "$1" = "uninstall" ] ; then
	uninstall $2 $pkg_type
elif [ "$1" = "addport" ] ; then
	iptables_add_port 
elif [ "$1" = "delport" ] ; then
	iptables_del_port 
fi

exit 0
